@@ -19,6 +19,8 @@
 //! encoding in Bitcoin script, as well as a datatype. Full details
 //! are given on the Miniscript website.
 
+use std::str::FromStr;
+use std::sync::Arc;
 use std::{fmt, str};
 
 use bitcoin::blockdata::{opcodes, script};
@@ -30,13 +32,7 @@ use expression;
 use miniscript::types::{self, Property};
 use miniscript::ScriptContext;
 use script_num_size;
-use std::sync::Arc;
-use str::FromStr;
-use Error;
-use Miniscript;
-use MiniscriptKey;
-use Terminal;
-use ToPublicKey;
+use {Error, ForEach, ForEachKey, Miniscript, MiniscriptKey, Terminal, ToPublicKey, TranslatePk};
 
 impl<Pk: MiniscriptKey, Ctx: ScriptContext> Terminal<Pk, Ctx> {
     /// Internal helper function for displaying wrapper types; returns
@@ -59,11 +55,72 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> Terminal<Pk, Ctx> {
     }
 }
 
-impl<Pk: MiniscriptKey, Ctx: ScriptContext> Terminal<Pk, Ctx> {
+impl<Pk: MiniscriptKey, Q: MiniscriptKey, Ctx: ScriptContext> TranslatePk<Pk, Q>
+    for Terminal<Pk, Ctx>
+{
+    type Output = Terminal<Q, Ctx>;
+
     /// Convert an AST element with one public key type to one of another
     /// public key type .This will panic while converting to
     /// Segwit Miniscript using uncompressed public keys
-    pub fn translate_pk<FPk, FPkh, Q, Error>(
+    fn translate_pk<FPk, FPkh, FuncError>(
+        &self,
+        mut translatefpk: FPk,
+        mut translatefpkh: FPkh,
+    ) -> Result<Self::Output, FuncError>
+    where
+        FPk: FnMut(&Pk) -> Result<Q, FuncError>,
+        FPkh: FnMut(&Pk::Hash) -> Result<Q::Hash, FuncError>,
+    {
+        self.real_translate_pk(&mut translatefpk, &mut translatefpkh)
+    }
+}
+
+impl<Pk: MiniscriptKey, Ctx: ScriptContext> Terminal<Pk, Ctx> {
+    pub(super) fn real_for_each_key<'a, F: FnMut(ForEach<'a, Pk>) -> bool>(
+        &'a self,
+        pred: &mut F,
+    ) -> bool
+    where
+        Pk: 'a,
+        Pk::Hash: 'a,
+    {
+        match *self {
+            Terminal::PkK(ref p) => pred(ForEach::Key(p)),
+            Terminal::PkH(ref p) => pred(ForEach::Hash(p)),
+            Terminal::After(..)
+            | Terminal::Older(..)
+            | Terminal::Sha256(..)
+            | Terminal::Hash256(..)
+            | Terminal::Ripemd160(..)
+            | Terminal::Hash160(..)
+            | Terminal::True
+            | Terminal::False => true,
+            Terminal::Alt(ref sub)
+            | Terminal::Swap(ref sub)
+            | Terminal::Check(ref sub)
+            | Terminal::DupIf(ref sub)
+            | Terminal::Verify(ref sub)
+            | Terminal::NonZero(ref sub)
+            | Terminal::ZeroNotEqual(ref sub) => sub.real_for_each_key(pred),
+            Terminal::AndV(ref left, ref right)
+            | Terminal::AndB(ref left, ref right)
+            | Terminal::OrB(ref left, ref right)
+            | Terminal::OrD(ref left, ref right)
+            | Terminal::OrC(ref left, ref right)
+            | Terminal::OrI(ref left, ref right) => {
+                left.real_for_each_key(&mut *pred) && right.real_for_each_key(pred)
+            }
+            Terminal::AndOr(ref a, ref b, ref c) => {
+                a.real_for_each_key(&mut *pred)
+                    && b.real_for_each_key(&mut *pred)
+                    && c.real_for_each_key(pred)
+            }
+            Terminal::Thresh(_, ref subs) => subs.iter().all(|sub| sub.real_for_each_key(pred)),
+            Terminal::Multi(_, ref keys) => keys.iter().all(|key| pred(ForEach::Key(key))),
+        }
+    }
+    pub(super) fn real_translate_pk<FPk, FPkh, Q, Error>(
         &self,
         translatefpk: &mut FPk,
         translatefpkh: &mut FPkh,
@@ -84,61 +141,63 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> Terminal<Pk, Ctx> {
             Terminal::Hash160(x) => Terminal::Hash160(x),
             Terminal::True => Terminal::True,
             Terminal::False => Terminal::False,
-            Terminal::Alt(ref sub) => {
-                Terminal::Alt(Arc::new(sub.translate_pk(translatefpk, translatefpkh)?))
-            }
-            Terminal::Swap(ref sub) => {
-                Terminal::Swap(Arc::new(sub.translate_pk(translatefpk, translatefpkh)?))
-            }
-            Terminal::Check(ref sub) => {
-                Terminal::Check(Arc::new(sub.translate_pk(translatefpk, translatefpkh)?))
-            }
-            Terminal::DupIf(ref sub) => {
-                Terminal::DupIf(Arc::new(sub.translate_pk(translatefpk, translatefpkh)?))
-            }
-            Terminal::Verify(ref sub) => {
-                Terminal::Verify(Arc::new(sub.translate_pk(translatefpk, translatefpkh)?))
-            }
-            Terminal::NonZero(ref sub) => {
-                Terminal::NonZero(Arc::new(sub.translate_pk(translatefpk, translatefpkh)?))
-            }
-            Terminal::ZeroNotEqual(ref sub) => {
-                Terminal::ZeroNotEqual(Arc::new(sub.translate_pk(translatefpk, translatefpkh)?))
-            }
+            Terminal::Alt(ref sub) => Terminal::Alt(Arc::new(
+                sub.real_translate_pk(translatefpk, translatefpkh)?,
+            )),
+            Terminal::Swap(ref sub) => Terminal::Swap(Arc::new(
+                sub.real_translate_pk(translatefpk, translatefpkh)?,
+            )),
+            Terminal::Check(ref sub) => Terminal::Check(Arc::new(
+                sub.real_translate_pk(translatefpk, translatefpkh)?,
+            )),
+            Terminal::DupIf(ref sub) => Terminal::DupIf(Arc::new(
+                sub.real_translate_pk(translatefpk, translatefpkh)?,
+            )),
+            Terminal::Verify(ref sub) => Terminal::Verify(Arc::new(
+                sub.real_translate_pk(translatefpk, translatefpkh)?,
+            )),
+            Terminal::NonZero(ref sub) => Terminal::NonZero(Arc::new(
+                sub.real_translate_pk(translatefpk, translatefpkh)?,
+            )),
+            Terminal::ZeroNotEqual(ref sub) => Terminal::ZeroNotEqual(Arc::new(
+                sub.real_translate_pk(translatefpk, translatefpkh)?,
+            )),
             Terminal::AndV(ref left, ref right) => Terminal::AndV(
-                Arc::new(left.translate_pk(&mut *translatefpk, &mut *translatefpkh)?),
-                Arc::new(right.translate_pk(translatefpk, translatefpkh)?),
+                Arc::new(left.real_translate_pk(&mut *translatefpk, &mut *translatefpkh)?),
+                Arc::new(right.real_translate_pk(translatefpk, translatefpkh)?),
             ),
             Terminal::AndB(ref left, ref right) => Terminal::AndB(
-                Arc::new(left.translate_pk(&mut *translatefpk, &mut *translatefpkh)?),
-                Arc::new(right.translate_pk(translatefpk, translatefpkh)?),
+                Arc::new(left.real_translate_pk(&mut *translatefpk, &mut *translatefpkh)?),
+                Arc::new(right.real_translate_pk(translatefpk, translatefpkh)?),
             ),
             Terminal::AndOr(ref a, ref b, ref c) => Terminal::AndOr(
-                Arc::new(a.translate_pk(&mut *translatefpk, &mut *translatefpkh)?),
-                Arc::new(b.translate_pk(&mut *translatefpk, &mut *translatefpkh)?),
-                Arc::new(c.translate_pk(translatefpk, translatefpkh)?),
+                Arc::new(a.real_translate_pk(&mut *translatefpk, &mut *translatefpkh)?),
+                Arc::new(b.real_translate_pk(&mut *translatefpk, &mut *translatefpkh)?),
+                Arc::new(c.real_translate_pk(translatefpk, translatefpkh)?),
             ),
             Terminal::OrB(ref left, ref right) => Terminal::OrB(
-                Arc::new(left.translate_pk(&mut *translatefpk, &mut *translatefpkh)?),
-                Arc::new(right.translate_pk(translatefpk, translatefpkh)?),
+                Arc::new(left.real_translate_pk(&mut *translatefpk, &mut *translatefpkh)?),
+                Arc::new(right.real_translate_pk(translatefpk, translatefpkh)?),
             ),
             Terminal::OrD(ref left, ref right) => Terminal::OrD(
-                Arc::new(left.translate_pk(&mut *translatefpk, &mut *translatefpkh)?),
-                Arc::new(right.translate_pk(translatefpk, translatefpkh)?),
+                Arc::new(left.real_translate_pk(&mut *translatefpk, &mut *translatefpkh)?),
+                Arc::new(right.real_translate_pk(translatefpk, translatefpkh)?),
             ),
             Terminal::OrC(ref left, ref right) => Terminal::OrC(
-                Arc::new(left.translate_pk(&mut *translatefpk, &mut *translatefpkh)?),
-                Arc::new(right.translate_pk(translatefpk, translatefpkh)?),
+                Arc::new(left.real_translate_pk(&mut *translatefpk, &mut *translatefpkh)?),
+                Arc::new(right.real_translate_pk(translatefpk, translatefpkh)?),
             ),
             Terminal::OrI(ref left, ref right) => Terminal::OrI(
-                Arc::new(left.translate_pk(&mut *&mut *translatefpk, &mut *&mut *translatefpkh)?),
-                Arc::new(right.translate_pk(translatefpk, translatefpkh)?),
+                Arc::new(
+                    left.real_translate_pk(&mut *&mut *translatefpk, &mut *&mut *translatefpkh)?,
+                ),
+                Arc::new(right.real_translate_pk(translatefpk, translatefpkh)?),
             ),
             Terminal::Thresh(k, ref subs) => {
                 let subs: Result<Vec<Arc<Miniscript<Q, _>>>, _> = subs
                     .iter()
                     .map(|s| {
-                        s.translate_pk(&mut *translatefpk, &mut *translatefpkh)
+                        s.real_translate_pk(&mut *translatefpk, &mut *translatefpkh)
                             .and_then(|x| Ok(Arc::new(x)))
                     })
                     .collect();
@@ -153,6 +212,16 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> Terminal<Pk, Ctx> {
     }
 }
 
+impl<Pk: MiniscriptKey, Ctx: ScriptContext> ForEachKey<Pk> for Terminal<Pk, Ctx> {
+    fn for_each_key<'a, F: FnMut(ForEach<'a, Pk>) -> bool>(&'a self, mut pred: F) -> bool
+    where
+        Pk: 'a,
+        Pk::Hash: 'a,
+    {
+        self.real_for_each_key(&mut pred)
+    }
+}
+
 impl<Pk: MiniscriptKey, Ctx: ScriptContext> fmt::Debug for Terminal<Pk, Ctx> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         f.write_str("[")?;
@@ -337,7 +406,8 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> fmt::Display for Terminal<Pk, Ctx> {
 
 impl<Pk, Ctx> expression::FromTree for Arc<Terminal<Pk, Ctx>>
 where
-    Pk: MiniscriptKey,
+    Pk: MiniscriptKey + str::FromStr,
+    Pk::Hash: str::FromStr,
     Ctx: ScriptContext,
     <Pk as str::FromStr>::Err: ToString,
     <<Pk as MiniscriptKey>::Hash as str::FromStr>::Err: ToString,
@@ -349,7 +419,8 @@ where
 
 impl<Pk, Ctx> expression::FromTree for Terminal<Pk, Ctx>
 where
-    Pk: MiniscriptKey,
+    Pk: MiniscriptKey + str::FromStr,
+    Pk::Hash: str::FromStr,
     Ctx: ScriptContext,
     <Pk as str::FromStr>::Err: ToString,
     <<Pk as MiniscriptKey>::Hash as str::FromStr>::Err: ToString,
@@ -550,17 +621,17 @@ where
 
 /// Helper trait to add a `push_astelem` method to `script::Builder`
 trait PushAstElem<Pk: MiniscriptKey, Ctx: ScriptContext> {
-    fn push_astelem<ToPkCtx: Copy>(self, ast: &Miniscript<Pk, Ctx>, to_pk_ctx: ToPkCtx) -> Self
+    fn push_astelem(self, ast: &Miniscript<Pk, Ctx>) -> Self
     where
-        Pk: ToPublicKey<ToPkCtx>;
+        Pk: ToPublicKey;
 }
 
 impl<Pk: MiniscriptKey, Ctx: ScriptContext> PushAstElem<Pk, Ctx> for script::Builder {
-    fn push_astelem<ToPkCtx: Copy>(self, ast: &Miniscript<Pk, Ctx>, to_pk_ctx: ToPkCtx) -> Self
+    fn push_astelem(self, ast: &Miniscript<Pk, Ctx>) -> Self
     where
-        Pk: ToPublicKey<ToPkCtx>,
+        Pk: ToPublicKey,
     {
-        ast.node.encode(self, to_pk_ctx)
+        ast.node.encode(self)
     }
 }
 
@@ -568,20 +639,16 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> Terminal<Pk, Ctx> {
     /// Encode the element as a fragment of Bitcoin Script. The inverse
     /// function, from Script to an AST element, is implemented in the
     /// `parse` module.
-    pub fn encode<ToPkCtx: Copy>(
-        &self,
-        mut builder: script::Builder,
-        to_pk_ctx: ToPkCtx,
-    ) -> script::Builder
+    pub fn encode(&self, mut builder: script::Builder) -> script::Builder
     where
-        Pk: ToPublicKey<ToPkCtx>,
+        Pk: ToPublicKey,
     {
         match *self {
-            Terminal::PkK(ref pk) => builder.push_key(&pk.to_public_key(to_pk_ctx)),
+            Terminal::PkK(ref pk) => builder.push_key(&pk.to_public_key()),
             Terminal::PkH(ref hash) => builder
                 .push_opcode(opcodes::all::OP_DUP)
                 .push_opcode(opcodes::all::OP_HASH160)
-                .push_slice(&Pk::hash_to_hash160(&hash, to_pk_ctx)[..])
+                .push_slice(&Pk::hash_to_hash160(&hash)[..])
                 .push_opcode(opcodes::all::OP_EQUALVERIFY),
             Terminal::After(t) => builder
                 .push_int(t as i64)
@@ -619,70 +686,64 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> Terminal<Pk, Ctx> {
             Terminal::False => builder.push_opcode(opcodes::OP_FALSE),
             Terminal::Alt(ref sub) => builder
                 .push_opcode(opcodes::all::OP_TOALTSTACK)
-                .push_astelem(sub, to_pk_ctx)
+                .push_astelem(sub)
                 .push_opcode(opcodes::all::OP_FROMALTSTACK),
-            Terminal::Swap(ref sub) => builder
-                .push_opcode(opcodes::all::OP_SWAP)
-                .push_astelem(sub, to_pk_ctx),
+            Terminal::Swap(ref sub) => builder.push_opcode(opcodes::all::OP_SWAP).push_astelem(sub),
             Terminal::Check(ref sub) => builder
-                .push_astelem(sub, to_pk_ctx)
+                .push_astelem(sub)
                 .push_opcode(opcodes::all::OP_CHECKSIG),
             Terminal::DupIf(ref sub) => builder
                 .push_opcode(opcodes::all::OP_DUP)
                 .push_opcode(opcodes::all::OP_IF)
-                .push_astelem(sub, to_pk_ctx)
+                .push_astelem(sub)
                 .push_opcode(opcodes::all::OP_ENDIF),
-            Terminal::Verify(ref sub) => builder.push_astelem(sub, to_pk_ctx).push_verify(),
+            Terminal::Verify(ref sub) => builder.push_astelem(sub).push_verify(),
             Terminal::NonZero(ref sub) => builder
                 .push_opcode(opcodes::all::OP_SIZE)
                 .push_opcode(opcodes::all::OP_0NOTEQUAL)
                 .push_opcode(opcodes::all::OP_IF)
-                .push_astelem(sub, to_pk_ctx)
+                .push_astelem(sub)
                 .push_opcode(opcodes::all::OP_ENDIF),
             Terminal::ZeroNotEqual(ref sub) => builder
-                .push_astelem(sub, to_pk_ctx)
+                .push_astelem(sub)
                 .push_opcode(opcodes::all::OP_0NOTEQUAL),
-            Terminal::AndV(ref left, ref right) => builder
-                .push_astelem(left, to_pk_ctx)
-                .push_astelem(right, to_pk_ctx),
+            Terminal::AndV(ref left, ref right) => builder.push_astelem(left).push_astelem(right),
             Terminal::AndB(ref left, ref right) => builder
-                .push_astelem(left, to_pk_ctx)
-                .push_astelem(right, to_pk_ctx)
+                .push_astelem(left)
+                .push_astelem(right)
                 .push_opcode(opcodes::all::OP_BOOLAND),
             Terminal::AndOr(ref a, ref b, ref c) => builder
-                .push_astelem(a, to_pk_ctx)
+                .push_astelem(a)
                 .push_opcode(opcodes::all::OP_NOTIF)
-                .push_astelem(c, to_pk_ctx)
+                .push_astelem(c)
                 .push_opcode(opcodes::all::OP_ELSE)
-                .push_astelem(b, to_pk_ctx)
+                .push_astelem(b)
                 .push_opcode(opcodes::all::OP_ENDIF),
             Terminal::OrB(ref left, ref right) => builder
-                .push_astelem(left, to_pk_ctx)
-                .push_astelem(right, to_pk_ctx)
+                .push_astelem(left)
+                .push_astelem(right)
                 .push_opcode(opcodes::all::OP_BOOLOR),
             Terminal::OrD(ref left, ref right) => builder
-                .push_astelem(left, to_pk_ctx)
+                .push_astelem(left)
                 .push_opcode(opcodes::all::OP_IFDUP)
                 .push_opcode(opcodes::all::OP_NOTIF)
-                .push_astelem(right, to_pk_ctx)
+                .push_astelem(right)
                 .push_opcode(opcodes::all::OP_ENDIF),
             Terminal::OrC(ref left, ref right) => builder
-                .push_astelem(left, to_pk_ctx)
+                .push_astelem(left)
                 .push_opcode(opcodes::all::OP_NOTIF)
-                .push_astelem(right, to_pk_ctx)
+                .push_astelem(right)
                 .push_opcode(opcodes::all::OP_ENDIF),
             Terminal::OrI(ref left, ref right) => builder
                 .push_opcode(opcodes::all::OP_IF)
-                .push_astelem(left, to_pk_ctx)
+                .push_astelem(left)
                 .push_opcode(opcodes::all::OP_ELSE)
-                .push_astelem(right, to_pk_ctx)
+                .push_astelem(right)
                 .push_opcode(opcodes::all::OP_ENDIF),
             Terminal::Thresh(k, ref subs) => {
-                builder = builder.push_astelem(&subs[0], to_pk_ctx);
+                builder = builder.push_astelem(&subs[0]);
                 for sub in &subs[1..] {
-                    builder = builder
-                        .push_astelem(sub, to_pk_ctx)
-                        .push_opcode(opcodes::all::OP_ADD);
+                    builder = builder.push_astelem(sub).push_opcode(opcodes::all::OP_ADD);
                 }
                 builder
                     .push_int(k as i64)
@@ -691,7 +752,7 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> Terminal<Pk, Ctx> {
             Terminal::Multi(k, ref keys) => {
                 builder = builder.push_int(k as i64);
                 for pk in keys {
-                    builder = builder.push_key(&pk.to_public_key(to_pk_ctx));
+                    builder = builder.push_key(&pk.to_public_key());
                 }
                 builder
                     .push_int(keys.len() as i64)
@@ -707,12 +768,9 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> Terminal<Pk, Ctx> {
     /// In general, it is not recommended to use this function directly, but
     /// to instead call the corresponding function on a `Descriptor`, which
     /// will handle the segwit/non-segwit technicalities for you.
-    pub fn script_size<ToPkCtx: Copy>(&self, to_pk_ctx: ToPkCtx) -> usize
-    where
-        Pk: ToPublicKey<ToPkCtx>,
-    {
+    pub fn script_size(&self) -> usize {
         match *self {
-            Terminal::PkK(ref pk) => pk.serialized_len(to_pk_ctx),
+            Terminal::PkK(ref pk) => pk.serialized_len(),
             Terminal::PkH(..) => 24,
             Terminal::After(n) => script_num_size(n as usize) + 1,
             Terminal::Older(n) => script_num_size(n as usize) + 1,
@@ -722,44 +780,29 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> Terminal<Pk, Ctx> {
             Terminal::Hash160(..) => 21 + 6,
             Terminal::True => 1,
             Terminal::False => 1,
-            Terminal::Alt(ref sub) => sub.node.script_size(to_pk_ctx) + 2,
-            Terminal::Swap(ref sub) => sub.node.script_size(to_pk_ctx) + 1,
-            Terminal::Check(ref sub) => sub.node.script_size(to_pk_ctx) + 1,
-            Terminal::DupIf(ref sub) => sub.node.script_size(to_pk_ctx) + 3,
+            Terminal::Alt(ref sub) => sub.node.script_size() + 2,
+            Terminal::Swap(ref sub) => sub.node.script_size() + 1,
+            Terminal::Check(ref sub) => sub.node.script_size() + 1,
+            Terminal::DupIf(ref sub) => sub.node.script_size() + 3,
             Terminal::Verify(ref sub) => {
-                sub.node.script_size(to_pk_ctx) + if sub.ext.has_free_verify { 0 } else { 1 }
-            }
-            Terminal::NonZero(ref sub) => sub.node.script_size(to_pk_ctx) + 4,
-            Terminal::ZeroNotEqual(ref sub) => sub.node.script_size(to_pk_ctx) + 1,
-            Terminal::AndV(ref l, ref r) => {
-                l.node.script_size(to_pk_ctx) + r.node.script_size(to_pk_ctx)
-            }
-            Terminal::AndB(ref l, ref r) => {
-                l.node.script_size(to_pk_ctx) + r.node.script_size(to_pk_ctx) + 1
+                sub.node.script_size() + if sub.ext.has_free_verify { 0 } else { 1 }
             }
+            Terminal::NonZero(ref sub) => sub.node.script_size() + 4,
+            Terminal::ZeroNotEqual(ref sub) => sub.node.script_size() + 1,
+            Terminal::AndV(ref l, ref r) => l.node.script_size() + r.node.script_size(),
+            Terminal::AndB(ref l, ref r) => l.node.script_size() + r.node.script_size() + 1,
             Terminal::AndOr(ref a, ref b, ref c) => {
-                a.node.script_size(to_pk_ctx)
-                    + b.node.script_size(to_pk_ctx)
-                    + c.node.script_size(to_pk_ctx)
-                    + 3
-            }
-            Terminal::OrB(ref l, ref r) => {
-                l.node.script_size(to_pk_ctx) + r.node.script_size(to_pk_ctx) + 1
-            }
-            Terminal::OrD(ref l, ref r) => {
-                l.node.script_size(to_pk_ctx) + r.node.script_size(to_pk_ctx) + 3
-            }
-            Terminal::OrC(ref l, ref r) => {
-                l.node.script_size(to_pk_ctx) + r.node.script_size(to_pk_ctx) + 2
-            }
-            Terminal::OrI(ref l, ref r) => {
-                l.node.script_size(to_pk_ctx) + r.node.script_size(to_pk_ctx) + 3
+                a.node.script_size() + b.node.script_size() + c.node.script_size() + 3
             }
+            Terminal::OrB(ref l, ref r) => l.node.script_size() + r.node.script_size() + 1,
+            Terminal::OrD(ref l, ref r) => l.node.script_size() + r.node.script_size() + 3,
+            Terminal::OrC(ref l, ref r) => l.node.script_size() + r.node.script_size() + 2,
+            Terminal::OrI(ref l, ref r) => l.node.script_size() + r.node.script_size() + 3,
             Terminal::Thresh(k, ref subs) => {
                 assert!(!subs.is_empty(), "threshold must be nonempty");
                 script_num_size(k) // k
                     + 1 // EQUAL
-                    + subs.iter().map(|s| s.node.script_size(to_pk_ctx)).sum::<usize>()
+                    + subs.iter().map(|s| s.node.script_size()).sum::<usize>()
                     + subs.len() // ADD
                     - 1 // no ADD on first element
             }
@@ -767,10 +810,7 @@ impl<Pk: MiniscriptKey, Ctx: ScriptContext> Terminal<Pk, Ctx> {
                 script_num_size(k)
                     + 1
                     + script_num_size(pks.len())
-                    + pks
-                        .iter()
-                        .map(|pk| pk.serialized_len(to_pk_ctx))
-                        .sum::<usize>()
+                    + pks.iter().map(|pk| pk.serialized_len()).sum::<usize>()
             }
         }
     }
